@@ -1,8 +1,10 @@
 /*
- * Copyright (C) 2014-2015 Fanout, Inc.
+ * Copyright (C) 2014-2017 Fanout, Inc.
  *
  * This file is part of Pushpin.
  *
+ * $FANOUT_BEGIN_LICENSE:AGPL$
+ *
  * Pushpin is free software: you can redistribute it and/or modify it under
  * the terms of the GNU Affero General Public License as published by the Free
  * Software Foundation, either version 3 of the License, or (at your option)
@@ -15,6 +17,13 @@
  *
  * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ *
+ * Alternatively, Pushpin may be used under the terms of a commercial license,
+ * where the commercial license agreement is provided with the software or
+ * contained in a written agreement between you and Fanout. For further
+ * information use the contact form at <https://fanout.io/enterprise/>.
+ *
+ * $FANOUT_END_LICENSE$
  */
 
 #ifndef STATSMANAGER_H
@@ -43,7 +52,10 @@ public:
 	void setInstanceId(const QByteArray &instanceId);
 	void setIpcFileMode(int mode);
 	bool setSpec(const QString &spec);
-	void setReportsEnabled(bool on);
+	void setConnectionTtl(int secs);
+	void setSubscriptionTtl(int secs);
+	void setSubscriptionLinger(int secs);
+	void setReportInterval(int secs);
 
 	// routeId may be empty for non-identified route
 
@@ -57,7 +69,7 @@ public:
 	//   send before removing with linger
 	void refreshConnection(const QByteArray &id);
 
-	void addSubscription(const QString &mode, const QString &channel);
+	void addSubscription(const QString &mode, const QString &channel, int subscriberCount);
 
 	// NOTE: may emit unsubscribed immediately (not DOR-DS)
 	void removeSubscription(const QString &mode, const QString &channel, bool linger);
