@@ -1,8 +1,10 @@
 /*
- * Copyright (C) 2015 Fanout, Inc.
+ * Copyright (C) 2015-2017 Fanout, Inc.
  *
  * This file is part of Pushpin.
  *
+ * $FANOUT_BEGIN_LICENSE:AGPL$
+ *
  * Pushpin is free software: you can redistribute it and/or modify it under
  * the terms of the GNU Affero General Public License as published by the Free
  * Software Foundation, either version 3 of the License, or (at your option)
@@ -15,6 +17,13 @@
  *
  * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ *
+ * Alternatively, Pushpin may be used under the terms of a commercial license,
+ * where the commercial license agreement is provided with the software or
+ * contained in a written agreement between you and Fanout. For further
+ * information use the contact form at <https://fanout.io/enterprise/>.
+ *
+ * $FANOUT_END_LICENSE$
  */
 
 #ifndef ENGINE_H
@@ -58,6 +67,12 @@ public:
 		int messageRate;
 		int messageHwm;
 		int messageBlockSize;
+		int idCacheTtl;
+		int connectionSubscriptionMax;
+		int subscriptionLinger;
+		int statsConnectionTtl;
+		int statsSubscriptionTtl;
+		int statsReportInterval;
 
 		Configuration() :
 			pushInHttpPort(-1),
@@ -65,7 +80,13 @@ public:
 			shareAll(false),
 			messageRate(-1),
 			messageHwm(-1),
-			messageBlockSize(-1)
+			messageBlockSize(-1),
+			idCacheTtl(-1),
+			connectionSubscriptionMax(-1),
+			subscriptionLinger(-1),
+			statsConnectionTtl(-1),
+			statsSubscriptionTtl(-1),
+			statsReportInterval(-1)
 		{
 		}
 	};
