@@ -3,6 +3,8 @@
  *
  * This file is part of Pushpin.
  *
+ * $FANOUT_BEGIN_LICENSE:AGPL$
+ *
  * Pushpin is free software: you can redistribute it and/or modify it under
  * the terms of the GNU Affero General Public License as published by the Free
  * Software Foundation, either version 3 of the License, or (at your option)
@@ -15,6 +17,13 @@
  *
  * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ *
+ * Alternatively, Pushpin may be used under the terms of a commercial license,
+ * where the commercial license agreement is provided with the software or
+ * contained in a written agreement between you and Fanout. For further
+ * information use the contact form at <https://fanout.io/enterprise/>.
+ *
+ * $FANOUT_END_LICENSE$
  */
 
 #include "httpsession.h"
@@ -165,8 +174,9 @@ public:
 	LogUtil::Config logConfig;
 	FilterStack *responseFilters;
 	QSet<QString> activeChannels;
+	int connectionSubscriptionMax;
 
-	Private(HttpSession *_q, ZhttpRequest *_req, const HttpSession::AcceptData &_adata, const Instruct &_instruct, ZhttpManager *_outZhttp, StatsManager *_stats, RateLimiter *_updateLimiter, PublishLastIds *_publishLastIds, HttpSessionUpdateManager *_updateManager) :
+	Private(HttpSession *_q, ZhttpRequest *_req, const HttpSession::AcceptData &_adata, const Instruct &_instruct, ZhttpManager *_outZhttp, StatsManager *_stats, RateLimiter *_updateLimiter, PublishLastIds *_publishLastIds, HttpSessionUpdateManager *_updateManager, int _connectionSubscriptionMax) :
 		QObject(_q),
 		q(_q),
 		req(_req),
@@ -181,7 +191,8 @@ public:
 		retries(0),
 		needUpdate(false),
 		pendingAction(0),
-		responseFilters(0)
+		responseFilters(0),
+		connectionSubscriptionMax(_connectionSubscriptionMax)
 	{
 		state = NotStarted;
 
@@ -244,6 +255,12 @@ public:
 			}
 		}
 
+		if(instruct.channels.count() > connectionSubscriptionMax)
+		{
+			instruct.channels = instruct.channels.mid(0, connectionSubscriptionMax);
+			log_warning("httpsession: too many subscriptions");
+		}
+
 		// need to send initial content?
 		if((instruct.holdMode == Instruct::NoHold || instruct.holdMode == Instruct::StreamHold) && !adata.responseSent)
 		{
@@ -396,6 +413,23 @@ public:
 				channel.prevId = item.id;
 			}
 
+			if(f.haveContentFilters)
+			{
+				// ensure content filters match
+				QStringList contentFilters;
+				foreach(const QString &f, channels[item.channel].filters)
+				{
+					if(Filter::isContentFilter(f))
+						contentFilters += f;
+				}
+				if(contentFilters != f.contentFilters)
+				{
+					errorMessage = QString("content filter mismatch: subscription=%1 message=%2").arg(contentFilters.join(","), f.contentFilters.join(","));
+					doError();
+					return;
+				}
+			}
+
 			QByteArray body;
 			if(f.haveBodyPatch)
 			{
@@ -606,8 +640,9 @@ private:
 			}
 			else
 			{
-				// update prev-id
+				// update channel properties
 				channels[name].prevId = c.prevId;
+				channels[name].filters = c.filters;
 			}
 		}
 
@@ -746,7 +781,22 @@ private:
 
 			PublishFormat &f = item.format;
 
-			QByteArray body = f.body;
+			if(f.haveContentFilters)
+			{
+				// ensure content filters match
+				QStringList contentFilters;
+				foreach(const QString &f, channels[item.channel].filters)
+				{
+					if(Filter::isContentFilter(f))
+						contentFilters += f;
+				}
+				if(contentFilters != f.contentFilters)
+				{
+					errorMessage = QString("content filter mismatch: subscription=%1 message=%2").arg(contentFilters.join(","), f.contentFilters.join(","));
+					doError();
+					break;
+				}
+			}
 
 			QHash<QString, QString> prevIds;
 			QHashIterator<QString, Instruct::Channel> it(channels);
@@ -767,16 +817,16 @@ private:
 			if(fs.sendAction() == Filter::Drop)
 				continue;
 
-			body = fs.process(body);
-			if(body.isNull())
-			{
-				errorMessage = QString("filter error: %1").arg(fs.errorMessage());
-				doError();
-				break;
-			}
-
 			if(f.action == PublishFormat::Send)
 			{
+				QByteArray body = fs.process(f.body);
+				if(body.isNull())
+				{
+					errorMessage = QString("filter error: %1").arg(fs.errorMessage());
+					doError();
+					break;
+				}
+
 				req->writeBody(body);
 
 				// restart keep alive timer
@@ -1225,6 +1275,12 @@ private:
 				else
 					nextUri.clear();
 
+				if(instruct.channels.count() > connectionSubscriptionMax)
+				{
+					instruct.channels = instruct.channels.mid(0, connectionSubscriptionMax);
+					log_warning("httpsession: too many subscriptions");
+				}
+
 				if(instruct.holdMode == Instruct::StreamHold)
 				{
 					if(instruct.keepAliveTimeout < 0)
@@ -1429,10 +1485,10 @@ private slots:
 	}
 };
 
-HttpSession::HttpSession(ZhttpRequest *req, const HttpSession::AcceptData &adata, const Instruct &instruct, ZhttpManager *zhttpOut, StatsManager *stats, RateLimiter *updateLimiter, PublishLastIds *publishLastIds, HttpSessionUpdateManager *updateManager, QObject *parent) :
+HttpSession::HttpSession(ZhttpRequest *req, const HttpSession::AcceptData &adata, const Instruct &instruct, ZhttpManager *zhttpOut, StatsManager *stats, RateLimiter *updateLimiter, PublishLastIds *publishLastIds, HttpSessionUpdateManager *updateManager, int connectionSubscriptionMax, QObject *parent) :
 	QObject(parent)
 {
-	d = new Private(this, req, adata, instruct, zhttpOut, stats, updateLimiter, publishLastIds, updateManager);
+	d = new Private(this, req, adata, instruct, zhttpOut, stats, updateLimiter, publishLastIds, updateManager, connectionSubscriptionMax);
 }
 
 HttpSession::~HttpSession()
