@@ -3,6 +3,8 @@
  *
  * This file is part of Pushpin.
  *
+ * $FANOUT_BEGIN_LICENSE:AGPL$
+ *
  * Pushpin is free software: you can redistribute it and/or modify it under
  * the terms of the GNU Affero General Public License as published by the Free
  * Software Foundation, either version 3 of the License, or (at your option)
@@ -15,6 +17,13 @@
  *
  * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ *
+ * Alternatively, Pushpin may be used under the terms of a commercial license,
+ * where the commercial license agreement is provided with the software or
+ * contained in a written agreement between you and Fanout. For further
+ * information use the contact form at <https://fanout.io/enterprise/>.
+ *
+ * $FANOUT_END_LICENSE$
  */
 
 #include "publishformat.h"
@@ -196,6 +205,32 @@ PublishFormat PublishFormat::fromVariant(Type type, const QVariant &in, bool *ok
 				}
 			}
 
+			if(keyedObjectContains(in, "content-filters"))
+			{
+				QVariant vfilters = keyedObjectGetValue(in, "content-filters");
+				if(vfilters.type() != QVariant::List)
+				{
+					setError(ok, errorMessage, QString("%1 contains 'content-filters' with wrong type").arg(pn));
+					return PublishFormat();
+				}
+
+				QStringList filters;
+				foreach(const QVariant &vfilter, vfilters.toList())
+				{
+					QString filter = getString(vfilter, &ok_);
+					if(!ok_)
+					{
+						setError(ok, errorMessage, "content-filters contains element with wrong type");
+						return PublishFormat();
+					}
+
+					filters += filter;
+				}
+
+				out.haveContentFilters = true;
+				out.contentFilters = filters;
+			}
+
 			if(in.type() == QVariant::Map && keyedObjectContains(in, "body-bin")) // JSON input
 			{
 				QString bodyBin = getString(in, pn, "body-bin", false, &ok_, errorMessage);
@@ -247,6 +282,32 @@ PublishFormat PublishFormat::fromVariant(Type type, const QVariant &in, bool *ok
 	{
 		if(out.action == Send)
 		{
+			if(keyedObjectContains(in, "content-filters"))
+			{
+				QVariant vfilters = keyedObjectGetValue(in, "content-filters");
+				if(vfilters.type() != QVariant::List)
+				{
+					setError(ok, errorMessage, QString("%1 contains 'content-filters' with wrong type").arg(pn));
+					return PublishFormat();
+				}
+
+				QStringList filters;
+				foreach(const QVariant &vfilter, vfilters.toList())
+				{
+					QString filter = getString(vfilter, &ok_);
+					if(!ok_)
+					{
+						setError(ok, errorMessage, "content-filters contains element with wrong type");
+						return PublishFormat();
+					}
+
+					filters += filter;
+				}
+
+				out.haveContentFilters = true;
+				out.contentFilters = filters;
+			}
+
 			if(in.type() == QVariant::Map && keyedObjectContains(in, "content-bin")) // JSON input
 			{
 				QString contentBin = getString(in, pn, "content-bin", false, &ok_, errorMessage);
