@@ -3,6 +3,8 @@
  *
  * This file is part of Pushpin.
  *
+ * $FANOUT_BEGIN_LICENSE:AGPL$
+ *
  * Pushpin is free software: you can redistribute it and/or modify it under
  * the terms of the GNU Affero General Public License as published by the Free
  * Software Foundation, either version 3 of the License, or (at your option)
@@ -15,6 +17,13 @@
  *
  * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ *
+ * Alternatively, Pushpin may be used under the terms of a commercial license,
+ * where the commercial license agreement is provided with the software or
+ * contained in a written agreement between you and Fanout. For further
+ * information use the contact form at <https://fanout.io/enterprise/>.
+ *
+ * $FANOUT_END_LICENSE$
  */
 
 #include "app.h"
@@ -88,9 +97,11 @@ public:
 	int logLevel;
 	QString ipcPrefix;
 	QStringList routeLines;
+	bool quietCheck;
 
 	ArgsData() :
-		logLevel(-1)
+		logLevel(-1),
+		quietCheck(false)
 	{
 	}
 };
@@ -110,6 +121,8 @@ static CommandLineParseResult parseCommandLine(QCommandLineParser *parser, ArgsD
 	parser->addOption(ipcPrefixOption);
 	const QCommandLineOption routeOption("route", "Add route (overrides routes file).", "line");
 	parser->addOption(routeOption);
+	const QCommandLineOption quietCheckOption("quiet-check", "Log update checks in Zurl as debug level.");
+	parser->addOption(quietCheckOption);
 	const QCommandLineOption helpOption = parser->addHelpOption();
 	const QCommandLineOption versionOption = parser->addVersionOption();
 
@@ -156,6 +169,9 @@ static CommandLineParseResult parseCommandLine(QCommandLineParser *parser, ArgsD
 			args->routeLines += r;
 	}
 
+	if(parser->isSet(quietCheckOption))
+		args->quietCheck = true;
+
 	return CommandLineOk;
 }
 
@@ -285,6 +301,7 @@ public:
 		QString sockJsUrl = settings.value("proxy/sockjs_url").toString();
 		QString updatesCheck = settings.value("proxy/updates_check").toString();
 		QString organizationName = settings.value("proxy/organization_name").toString();
+		int statsConnectionTtl = settings.value("global/stats_connection_ttl", 120).toInt();
 
 		QList<QByteArray> origHeadersNeedMark;
 		foreach(const QString &s, origHeadersNeedMarkStr)
@@ -345,6 +362,8 @@ public:
 		config.sockJsUrl = sockJsUrl;
 		config.updatesCheck = updatesCheck;
 		config.organizationName = organizationName;
+		config.quietCheck = args.quietCheck;
+		config.statsConnectionTtl = statsConnectionTtl;
 
 		engine = new Engine(this);
 		if(!engine->start(config))
