@@ -3,6 +3,8 @@
  *
  * This file is part of Pushpin.
  *
+ * $FANOUT_BEGIN_LICENSE:AGPL$
+ *
  * Pushpin is free software: you can redistribute it and/or modify it under
  * the terms of the GNU Affero General Public License as published by the Free
  * Software Foundation, either version 3 of the License, or (at your option)
@@ -15,6 +17,13 @@
  *
  * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ *
+ * Alternatively, Pushpin may be used under the terms of a commercial license,
+ * where the commercial license agreement is provided with the software or
+ * contained in a written agreement between you and Fanout. For further
+ * information use the contact form at <https://fanout.io/enterprise/>.
+ *
+ * $FANOUT_END_LICENSE$
  */
 
 #include "engine.h"
@@ -297,6 +306,8 @@ public:
 				log_error("unable to bind to stats_spec: %s", qPrintable(config.statsSpec));
 				return false;
 			}
+
+			stats->setConnectionTtl(config.statsConnectionTtl);
 		}
 
 		if(!config.commandSpec.isEmpty())
@@ -315,7 +326,7 @@ public:
 
 		if(!config.appVersion.isEmpty() && (config.updatesCheck == "check" || config.updatesCheck == "report"))
 		{
-			updater = new Updater(config.updatesCheck == "report" ? Updater::ReportMode : Updater::CheckMode, config.appVersion, config.organizationName, zroutes->defaultManager(), this);
+			updater = new Updater(config.updatesCheck == "report" ? Updater::ReportMode : Updater::CheckMode, config.quietCheck, config.appVersion, config.organizationName, zroutes->defaultManager(), this);
 		}
 
 		// init zroutes
